@@ -1,11 +1,11 @@
+#include "dat.h"
+#include <errno.h>
+#include <limits.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>
-#include <limits.h>
 #include <unistd.h>
-#include "dat.h"
 
 #define SAFETY_MARGIN (1000000000) /* 1 second */
 
@@ -14,32 +14,37 @@ static uint tot_conn_ct = 0;
 int verbose = 0;
 
 static void
-on_watch(ms a, tube t, size_t i)
+on_watch(Ms *a, Tube *t, size_t i)
 {
+    UNUSED_PARAMETER(a);
+    UNUSED_PARAMETER(i);
     tube_iref(t);
     t->watching_ct++;
 }
 
 static void
-on_ignore(ms a, tube t, size_t i)
+on_ignore(Ms *a, Tube *t, size_t i)
 {
+    UNUSED_PARAMETER(a);
+    UNUSED_PARAMETER(i);
     t->watching_ct--;
     tube_dref(t);
 }
 
 Conn *
-make_conn(int fd, char start_state, tube use, tube watch)
+make_conn(int fd, char start_state, Tube *use, Tube *watch)
 {
-    job j;
-    Conn *c;
-
-    c = new(Conn);
-    if (!c) return twarn("OOM"), NULL;
+    Conn *c = new(Conn);
+    if (!c) {
+        twarn("OOM");
+        return NULL;
+    }
 
     ms_init(&c->watch, (ms_event_fn) on_watch, (ms_event_fn) on_ignore);
     if (!ms_append(&c->watch, watch)) {
         free(c);
-        return twarn("OOM"), NULL;
+        twarn("OOM");
+        return NULL;
     }
 
     TUBE_ASSIGN(c->use, use);
@@ -48,9 +53,11 @@ make_conn(int fd, char start_state, tube use, tube watch)
     c->sock.fd = fd;
     c->state = start_state;
     c->pending_timeout = -1;
-    c->tickpos = -1;
-    j = &c->reserved_jobs;
-    j->prev = j->next = j;
+    c->tickpos = 0; // Does not mean anything if in_conns is set to 0.
+    c->in_conns = 0;
+
+    // The list is empty.
+    job_list_reset(&c->reserved_jobs);
 
     /* stats */
     cur_conn_ct++;
@@ -102,10 +109,11 @@ count_cur_workers()
 static int
 has_reserved_job(Conn *c)
 {
-    return job_list_any_p(&c->reserved_jobs);
+    return !job_list_is_empty(&c->reserved_jobs);
 }
 
 
+// Returns positive nanoseconds when c should tick, 0 otherwise.
 static int64
 conntickat(Conn *c)
 {
@@ -132,52 +140,67 @@ conntickat(Conn *c)
 }
 
 
-void
-connwant(Conn *c, int rw)
-{
-    c->rw = rw;
-    connsched(c);
-}
-
-
+// Remove c from the c->srv heap and reschedule it using the value
+// returned by conntickat if there is an outstanding timeout in the c.
 void
 connsched(Conn *c)
 {
-    if (c->tickpos > -1) {
+    if (c->in_conns) {
         heapremove(&c->srv->conns, c->tickpos);
+        c->in_conns = 0;
     }
     c->tickat = conntickat(c);
     if (c->tickat) {
         heapinsert(&c->srv->conns, c);
+        c->in_conns = 1;
     }
 }
 
+// conn_set_soonestjob updates c->soonest_job with j
+// if j should be handled sooner than c->soonest_job.
+static void
+conn_set_soonestjob(Conn *c, Job *j) {
+    if (!c->soonest_job || j->r.deadline_at < c->soonest_job->r.deadline_at) {
+        c->soonest_job = j;
+    }
+}
 
-/* return the reserved job with the earliest deadline,
- * or NULL if there's no reserved job */
-job
+// Return the reserved job with the earliest deadline,
+// or NULL if there's no reserved job.
+Job *
 connsoonestjob(Conn *c)
 {
-    job j = NULL;
-    job soonest = c->soonest_job;
+    // use cached value and bail out.
+    if (c->soonest_job != NULL)
+        return c->soonest_job;
 
-    if (soonest == NULL) {
-        for (j = c->reserved_jobs.next; j != &c->reserved_jobs; j = j->next) {
-            if (j->r.deadline_at <= (soonest ? : j)->r.deadline_at) soonest = j;
-        }
+    Job *j = NULL;
+    for (j = c->reserved_jobs.next; j != &c->reserved_jobs; j = j->next) {
+        conn_set_soonestjob(c, j);
     }
-    c->soonest_job = soonest;
-    return soonest;
+    return c->soonest_job;
 }
 
+void
+conn_reserve_job(Conn *c, Job *j) {
+    j->tube->stat.reserved_ct++;
+    j->r.reserve_ct++;
 
-/* return true if c has a reserved job with less than one second until its
- * deadline */
+    j->r.deadline_at = nanoseconds() + j->r.ttr;
+    j->r.state = Reserved;
+    job_list_insert(&c->reserved_jobs, j);
+    j->reserver = c;
+    c->pending_timeout = -1;
+    conn_set_soonestjob(c, j);
+}
+
+// Return true if c has a reserved job with less than one second until its
+// deadline.
 int
 conndeadlinesoon(Conn *c)
 {
     int64 t = nanoseconds();
-    job j = connsoonestjob(c);
+    Job *j = connsoonestjob(c);
 
     return j && t >= j->r.deadline_at - SAFETY_MARGIN;
 }
@@ -187,24 +210,27 @@ conn_ready(Conn *c)
 {
     size_t i;
 
-    for (i = 0; i < c->watch.used; i++) {
-        if (((tube) c->watch.items[i])->ready.len) return 1;
+    for (i = 0; i < c->watch.len; i++) {
+        if (((Tube *) c->watch.items[i])->ready.len)
+            return 1;
     }
     return 0;
 }
 
 
 int
-connless(Conn *a, Conn *b)
+conn_less(void *ca, void *cb)
 {
+    Conn *a = (Conn *)ca;
+    Conn *b = (Conn *)cb;
     return a->tickat < b->tickat;
 }
 
 
 void
-connrec(Conn *c, int i)
+conn_setpos(void *c, size_t i)
 {
-    c->tickpos = i;
+    ((Conn *)c)->tickpos = i;
 }
 
 
@@ -220,7 +246,8 @@ connclose(Conn *c)
     job_free(c->in_job);
 
     /* was this a peek or stats command? */
-    if (c->out_job && !c->out_job->r.id) job_free(c->out_job);
+    if (c->out_job && c->out_job->r.state == Copy)
+        job_free(c->out_job);
 
     c->in_job = c->out_job = NULL;
     c->in_job_read = 0;
@@ -231,14 +258,16 @@ connclose(Conn *c)
     cur_conn_ct--; /* stats */
 
     remove_waiting_conn(c);
-    if (has_reserved_job(c)) enqueue_reserved_jobs(c);
+    if (has_reserved_job(c))
+        enqueue_reserved_jobs(c);
 
     ms_clear(&c->watch);
     c->use->using_ct--;
     TUBE_ASSIGN(c->use, NULL);
 
-    if (c->tickpos > -1) {
+    if (c->in_conns) {
         heapremove(&c->srv->conns, c->tickpos);
+        c->in_conns = 0;
     }
 
     free(c);
