@@ -1,4 +1,6 @@
 #define _XOPEN_SOURCE 600
+
+#include "dat.h"
 #include <unistd.h>
 #include <sys/types.h>
 #include <stdint.h>
@@ -6,7 +8,6 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <sys/epoll.h>
-#include "dat.h"
 
 #ifndef EPOLLRDHUP
 #define EPOLLRDHUP 0x2000
@@ -15,16 +16,6 @@
 static int epfd;
 
 
-/* Allocate disk space.
- * Expects fd's offset to be 0; may also reset fd's offset to 0.
- * Returns 0 on success, and a positive errno otherwise. */
-int
-rawfalloc(int fd, int len)
-{
-    return ftruncate(fd, len);
-}
-
-
 int
 sockinit(void)
 {
@@ -41,7 +32,6 @@ int
 sockwant(Socket *s, int rw)
 {
     int op;
-    struct epoll_event ev = {};
 
     if (!s->added && !rw) {
         return 0;
@@ -54,6 +44,7 @@ sockwant(Socket *s, int rw)
         op = EPOLL_CTL_MOD;
     }
 
+    struct epoll_event ev = {.events=0};
     switch (rw) {
     case 'r':
         ev.events = EPOLLIN;
@@ -73,7 +64,7 @@ int
 socknext(Socket **s, int64 timeout)
 {
     int r;
-    struct epoll_event ev;
+    struct epoll_event ev = {.events=0};
 
     r = epoll_wait(epfd, &ev, 1, (int)(timeout/1000000));
     if (r == -1 && errno != EINTR) {
